using System;
using System.Xml.Serialization;
using MediaPortal.GUI.Library;
using MediaPortal.Util;

// need when using a dialog box or the virtual keyboard
using MediaPortal.GUI.Library;
using MediaPortal.Dialogs;
using MediaPortal.Player;

using Microsoft.Office.Core;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using DSOFile;

namespace MediaPortal.Plugins.MySlideShows
{
	/// <summary>
	/// Summary description for MySlideShowInfo.
	/// </summary>
	public class MySlideShowInfo : GUIWindow
	{
		#region Construction & cleanup

		public MySlideShowInfo()
		{
			GetID = 30001;
		}

		~MySlideShowInfo()
		{
		}

		#endregion Construction & cleanup

		#region Serialization

		[Serializable]
			public class Settings
		{
			public Settings()
			{
			}

			[XmlElement("ViewSystem")]
			public string Viewer
			{ get { return m_strView; } set { m_strView = value; } }

			[XmlElement("PowerPointViewerDirectory")]
			public string PowerPointViewerDirectory
			{ get { return m_strPPTViewDir; } set { m_strPPTViewDir = value; }	}

			public void Load()
			{
				using(MediaPortal.Profile.Xml xmlreader = new MediaPortal.Profile.Xml("MediaPortal.xml"))
				{
					m_strView = xmlreader.GetValueAsString("MySlideshows","ViewSystem","Office2003");
					m_strPPTViewDir = xmlreader.GetValueAsString("MySlideshows","PowerPointViewerDirectory",
						"C:\\Program Files\\Microsoft Office\\PowerPoint Viewer");
				}
			}

			// TODO: Declare your member variables for each setting/property
			protected string		m_strView = "Office2003";
			protected string		m_strPPTViewDir = "C:\\Program Files\\Microsoft Office\\PowerPoint Viewer";
		}

		#endregion Serialization
		
		#region Helper Enums

		// TODO: declare meaningful names for window's controls here
		enum Controls
		{
			LABEL_TITLE					= 3,
			LABEL_DATE					= 5,
			LABEL_MADEBY				= 7,
			TEXTBOXUPSCROLL_RESUME		= 10,
			BUTTON_START				= 11
		};

		// TODO: declare meaningful names for the strings used by the window
		enum Strings
		{
			NOT_AVAILABLE				= 161
		}

		enum ViewSystem
		{
			Office2003,
			OpenOffice,
			PowerPointViewer
		};

		#endregion Helper Enums

		#region Overrides

		public override bool Init()
		{
			bool result =  Load(GUIGraphicsContext.Skin + @"\myslideshowinfo.xml");
			return result;
		}

		public override void OnAction(Action action)
		{
			if(action.wID == Action.ActionType.ACTION_KEY_PRESSED)
			{
				int key = action.m_key.KeyCode;
				if (key == (int)Keys.Enter)
				{
					switch(GetFocusControlId())
					{
						case (int)Controls.BUTTON_START:
						{
							if (mViewSystem == ViewSystem.Office2003)
							{
								StartPresentationWithPPT();
							}
							else if (mViewSystem == ViewSystem.PowerPointViewer)
							{	
								StartPresentationWithPPTViewer();
							}
							break;
						}
					}
				}
			}
			else
			{
				base.OnAction(action);
			}
		}

		public override bool OnMessage(GUIMessage message)
		{
			switch(message.Message )
			{
				case GUIMessage.MessageType.GUI_MSG_WINDOW_INIT:
					base.OnMessage(message);

					OnInit();
					
					return true;
				case GUIMessage.MessageType.GUI_MSG_WINDOW_DEINIT:

					OnDeinit();

					break;
				case GUIMessage.MessageType.GUI_MSG_CLICKED:
					base.OnMessage(message);
					if (message.SenderControlId == (int)Controls.BUTTON_START)
					{
						if (mViewSystem == ViewSystem.Office2003)
						{
							StartPresentationWithPPT();
						}
						else if (mViewSystem == ViewSystem.PowerPointViewer)
						{	
							StartPresentationWithPPTViewer();
						}
					}
					break;
				default:
					return base.OnMessage(message);
			}

			return true;
		}


		#endregion Overrides

		#region imports

		[DllImport("user32")]
		private static extern bool SetForegroundWindow(IntPtr hWnd);
		
		[DllImport("user32")]
		private static extern IntPtr GetForegroundWindow();

		[DllImport("user32")]
		private static extern bool BringWindowToTop(IntPtr hWnd);

		[DllImport("user32")] 
		private static extern bool ShowWindowAsync(IntPtr hWnd, int nCmdShow);

		[DllImport("user32")]
		private static extern bool AttachThreadInput(int nThreadId, int nThreadIdTo, bool bAttach);
		
		[DllImport("user32")]
		private static extern int GetWindowThreadProcessId(IntPtr hWnd, int unused);

		[DllImport("user32")]
		static extern bool IsWindowVisible(IntPtr hWnd);

		[DllImport("user32")]
		static extern bool IsIconic(IntPtr hWnd);

		#endregion

		#region Implementation

		static bool SetForegroundWindow(IntPtr window, bool force)
		{
			IntPtr windowForeground = GetForegroundWindow(); 

			if(window == windowForeground || SetForegroundWindow(window))
				return true;

			if(force == false)
				return false;
			
			if(windowForeground == IntPtr.Zero)
				return false;

			// if we don't attach successfully to the windows thread then we're out of options
			if(!AttachThreadInput(AppDomain.GetCurrentThreadId(), GetWindowThreadProcessId(windowForeground, 0), true))
				return false;

			SetForegroundWindow(window);
			BringWindowToTop(window);

			AttachThreadInput(AppDomain.GetCurrentThreadId(), GetWindowThreadProcessId(windowForeground, 0), false);

			// we've done all that we can so base our return value on whether we have succeeded or not
			return (GetForegroundWindow() == window);
		}

		void OnInit()
		{
			_Settings.Load();
			mViewSystem = StringToViewSystem(_Settings.Viewer);

			txtResume = GetControl((int)Controls.TEXTBOXUPSCROLL_RESUME) as GUITextScrollUpControl;
			lblMadeBy = GetControl((int)Controls.LABEL_MADEBY) as GUILabelControl;
			lblDate = GetControl((int)Controls.LABEL_DATE) as GUILabelControl;
			lblTitle = GetControl((int)Controls.LABEL_TITLE) as GUILabelControl;

			// Try needed or it'll crash the first time you use it!
			try
			{
				DSOFile.OleDocumentPropertiesClass m_oDocument = new DSOFile.OleDocumentPropertiesClass();
				m_oDocument.Open(MySlideShows.currentFile, true, DSOFile.dsoFileOpenOptions.dsoOptionOpenReadOnlyIfNoWriteAccess);
				DSOFile.SummaryProperties oSummProps = m_oDocument.SummaryProperties;
				lblMadeBy.Label	= oSummProps.Author;
				lblDate.Label	= oSummProps.DateCreated.ToString();
				lblTitle.Label	= oSummProps.Title;
				txtResume.Label = oSummProps.Comments;
				m_oDocument.Close(false);
			}
			catch (Exception e){
				Log.Write("Error {0} at Source {1}\n{2}",e.Message,e.Source,e.StackTrace);
			}

		}

		void OnDeinit()
		{
		}

		string GetProperty(string prop)
		{
			object _props = ppPres.BuiltInDocumentProperties;
			object _p = _props.GetType().InvokeMember("Item",BindingFlags.GetProperty,null,_props,new object[]{ prop });
			object _value = _p.GetType().InvokeMember("Value", BindingFlags.GetProperty, null, _p,new object[] {} );
			if(_value == null)
				return GUILocalizeStrings.Get((int)Strings.NOT_AVAILABLE);
			return _value.ToString();
		}

		void StartPresentationWithPPT()
		{
			g_Player.Pause();
			ppApp = new Microsoft.Office.Interop.PowerPoint.Application();
			ppPres = ppApp.Presentations.Open(MySlideShows.currentFile,MsoTriState.msoTrue, MsoTriState.msoFalse,MsoTriState.msoFalse);
			ppPres.SlideShowSettings.Run();
			SetForegroundWindow((IntPtr)ppPres.SlideShowWindow.HWND,true);
			// QI for IConnectionPointContainer.
			UCOMIConnectionPointContainer oConnPointContainer = (UCOMIConnectionPointContainer) ppApp;
			// Get the GUID of the EApplication interface.
			Guid guid=typeof(PowerPoint.EApplication).GUID;

			// Find the connection point.
			oConnPointContainer.FindConnectionPoint(ref guid,out m_oConnectionPoint);
			// Call Advise to sink up the connection.
			m_oConnectionPoint.Advise(this,out m_Cookie);
		}

		void StartPresentationWithPPTViewer()
		{
			g_Player.Pause();
			System.Diagnostics.ProcessStartInfo proc = new System.Diagnostics.ProcessStartInfo();
			proc.FileName = _Settings.PowerPointViewerDirectory + @"\\pptview.exe";
			proc.Arguments = "/S \"" + MySlideShows.currentFile + "\"";
			Utils.StartProcess(proc,true);
			g_Player.Pause();
		}

		ViewSystem StringToViewSystem(string strView)
		{
			if (strView == "Office2003")
				return ViewSystem.Office2003;
			if (strView == "PowerPointViewer")
				return ViewSystem.PowerPointViewer;
			if (strView == "OpenOffice")
				return ViewSystem.OpenOffice;
			return ViewSystem.PowerPointViewer;
		}

		[DispId(2014)]
		public void SlideShowEnd(PowerPoint.Presentation Pres)
		{
			SetForegroundWindow(GUIGraphicsContext.ActiveForm,true);
			m_oConnectionPoint.Unadvise(m_Cookie);
			ppPres.Close();
			System.Runtime.InteropServices.Marshal.ReleaseComObject(ppApp);
			GC.Collect();
			g_Player.Pause();
		}	

		#endregion Implementation

		#region Member Variables	
		Microsoft.Office.Interop.PowerPoint.Application ppApp;
		Microsoft.Office.Interop.PowerPoint._Presentation ppPres;
		bool useDSOFile;
		private UCOMIConnectionPoint m_oConnectionPoint;
		private int m_Cookie;
		Settings _Settings = new Settings();
		ViewSystem mViewSystem;

		[SkinControlAttribute(3)] protected GUILabelControl lblTitle = null;
		[SkinControlAttribute(5)] protected GUILabelControl lblDate = null;
		[SkinControlAttribute(7)] protected GUILabelControl lblMadeBy = null;
		[SkinControlAttribute(10)] protected GUITextScrollUpControl txtResume = null;
		#endregion Member Variables
	}
}
